import hashlib
from .b64 import decode as b64_decode 
from .b64 import encode as b64_encode
from cryptography.hazmat.primitives import hashes, serialization

def get_key_hash(key):
    public_key = key.public_key()
    spki = public_key.public_bytes(
        encoding=serialization.Encoding.DER,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )
    digest = hashes.Hash(hashes.SHA256())
    digest.update(spki)

    hashed = digest.finalize()
    base64url_hash = b64_encode(hashed)
    return base64url_hash


def get_chunk_hash(chunk_data):
    sha256_hash = hashlib.sha256(chunk_data).digest()
    return b64_encode(sha256_hash)


def get_full_chunk_hash(hash_list):
    hash = hashlib.sha256()
    for b64 in hash_list:
        buffer = b64_decode(b64)
        hash.update(buffer)
    return b64_encode(hash.digest())



import pytest
from click.testing import CliRunner
from dabih.main import main
from dabih.config import get_client
import dabih.config

@pytest.fixture
def runner():
    return CliRunner()

# def test_token_invalid(runner, monkeypatch):
#     original_get_client = get_client
#     def mock_get_client():
#         client, pem_files = original_get_client()
#         client.token = 'kjhbdfkahs'
#         return client, pem_files
#     monkeypatch.setattr('dabih.config.get_client', mock_get_client)

#     result = runner.invoke(main, ['token-val'])
#     print(result.output)
#     assert 'Token is not valid' in result.output

# def test_invalid_base_url(runner, monkeypatch):
#     original_get_client = get_client
#     def mock_get_client():
#         client, pem_files = original_get_client()
#         client.base_url = 'http://invalid-url'
#         return client, pem_files
#     monkeypatch.setattr('dabih.config.get_client', mock_get_client)

#     result = runner.invoke(main, ['check'])
#     print(result.output)
#     assert 'Connection error: Please check the URL in config file or whether the server is running' in result.output


def test_upload_with_invalid_file(runner):
    result = runner.invoke(main, ['upload', 'non_existent_file.txt'])
    print(result.output)
    assert 'File at path: non_existent_file.txt not found. Please check the path.' in result.output

def test_upload_with_invalid_folder(runner):
    result = runner.invoke(main, ['upload', 'test.txt', 'invalid_folder_mnemonic'])
    print(result.output)
    assert 'Requested dabih file/folder not found: Inode invalid_folder_mnemonic not found.' in result.output

def test_download_with_invalid_mnemonic(runner):
    result = runner.invoke(main, ['download', 'invalid_mnemonic'])
    print(result.output)
    assert 'Requested dabih file/folder not found: No file found for mnemonic invalid_mnemonic.' in result.output



import datetime
from typing import Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

T = TypeVar("T", bound="Chunk")


@_attrs_define
class Chunk:
    """
    Attributes:
        id (Any): The database id of the chunk
        data_id (Any): The id of the data the chunk belongs to
        hash_ (str): The SHA-256 hash of the unencrypted chunk data base64url encoded
        iv (str): The AES-256 initialization vector base64url encoded
        start (Any): The start of the chunk as a byte position in the file
        end (Any): The end of the chunk as a byte position in the file
        crc (Union[None, str]): The CRC32 checksum of the encrypted chunk data base64url encoded
        created_at (datetime.datetime): chunk creation timestamp
        updated_at (datetime.datetime): chunk last update timestamp
    """

    id: Any
    data_id: Any
    hash_: str
    iv: str
    start: Any
    end: Any
    crc: Union[None, str]
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        data_id = self.data_id

        hash_ = self.hash_

        iv = self.iv

        start = self.start

        end = self.end

        crc: Union[None, str]
        crc = self.crc

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "dataId": data_id,
                "hash": hash_,
                "iv": iv,
                "start": start,
                "end": end,
                "crc": crc,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        data_id = d.pop("dataId")

        hash_ = d.pop("hash")

        iv = d.pop("iv")

        start = d.pop("start")

        end = d.pop("end")

        def _parse_crc(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        crc = _parse_crc(d.pop("crc"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        chunk = cls(
            id=id,
            data_id=data_id,
            hash_=hash_,
            iv=iv,
            start=start,
            end=end,
            crc=crc,
            created_at=created_at,
            updated_at=updated_at,
        )

        return chunk



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.file_decryption_key import FileDecryptionKey


T = TypeVar("T", bound="MemberAddBody")


@_attrs_define
class MemberAddBody:
    """
    Attributes:
        subs (List[str]): The users to add to the dataset
        keys (List['FileDecryptionKey']): The list of AES-256 keys required to decrypt all child datasets
    """

    subs: List[str]
    keys: List["FileDecryptionKey"]

    def to_dict(self) -> Dict[str, Any]:
        subs = self.subs

        keys = []
        for keys_item_data in self.keys:
            keys_item = keys_item_data.to_dict()
            keys.append(keys_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "subs": subs,
                "keys": keys,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.file_decryption_key import FileDecryptionKey

        d = src_dict.copy()
        subs = cast(List[str], d.pop("subs"))

        keys = []
        _keys = d.pop("keys")
        for keys_item_data in _keys:
            keys_item = FileDecryptionKey.from_dict(keys_item_data)

            keys.append(keys_item)

        member_add_body = cls(
            subs=subs,
            keys=keys,
        )

        return member_add_body



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.public_key import PublicKey


T = TypeVar("T", bound="UserResponse")


@_attrs_define
class UserResponse:
    """
    Attributes:
        id (Any): The database id of the user
        sub (str): The unique user sub
        name (str): The name of the user
        email (str): The email of the user
        created_at (datetime.datetime): The date the user was created
        updated_at (datetime.datetime): The date the user was last updated
        keys (List['PublicKey']): The public keys of the user
    """

    id: Any
    sub: str
    name: str
    email: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    keys: List["PublicKey"]

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        sub = self.sub

        name = self.name

        email = self.email

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        keys = []
        for keys_item_data in self.keys:
            keys_item = keys_item_data.to_dict()
            keys.append(keys_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "sub": sub,
                "name": name,
                "email": email,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "keys": keys,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.public_key import PublicKey

        d = src_dict.copy()
        id = d.pop("id")

        sub = d.pop("sub")

        name = d.pop("name")

        email = d.pop("email")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        keys = []
        _keys = d.pop("keys")
        for keys_item_data in _keys:
            keys_item = PublicKey.from_dict(keys_item_data)

            keys.append(keys_item)

        user_response = cls(
            id=id,
            sub=sub,
            name=name,
            email=email,
            created_at=created_at,
            updated_at=updated_at,
            keys=keys,
        )

        return user_response



from .download import *
from .filesystem import *
from .job import *
from .token import *
from .upload import *
from .users import *
from .util import *
from .json import *
from .crypto.hash import *



import datetime
from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

T = TypeVar("T", bound="Directory")


@_attrs_define
class Directory:
    """
    Attributes:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        name (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    mnemonic: str
    name: str
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        mnemonic = self.mnemonic

        name = self.name

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "mnemonic": mnemonic,
                "name": name,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        mnemonic = d.pop("mnemonic")

        name = d.pop("name")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        directory = cls(
            mnemonic=mnemonic,
            name=name,
            created_at=created_at,
            updated_at=updated_at,
        )

        return directory



import datetime
from typing import Any, Dict, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from dateutil.parser import isoparse

T = TypeVar("T", bound="PublicKey")


@_attrs_define
class PublicKey:
    """
    Attributes:
        id (Any): The database id of the public key
        user_id (Any): The user id the key belongs to
        hash_ (str):
        data (str):
        is_root_key (bool):
        enabled (Union[None, datetime.datetime]):
        enabled_by (Union[None, str]):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    id: Any
    user_id: Any
    hash_: str
    data: str
    is_root_key: bool
    enabled: Union[None, datetime.datetime]
    enabled_by: Union[None, str]
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        user_id = self.user_id

        hash_ = self.hash_

        data = self.data

        is_root_key = self.is_root_key

        enabled: Union[None, str]
        if isinstance(self.enabled, datetime.datetime):
            enabled = self.enabled.isoformat()
        else:
            enabled = self.enabled

        enabled_by: Union[None, str]
        enabled_by = self.enabled_by

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "userId": user_id,
                "hash": hash_,
                "data": data,
                "isRootKey": is_root_key,
                "enabled": enabled,
                "enabledBy": enabled_by,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        user_id = d.pop("userId")

        hash_ = d.pop("hash")

        data = d.pop("data")

        is_root_key = d.pop("isRootKey")

        def _parse_enabled(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                enabled_type_0 = isoparse(data)

                return enabled_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        enabled = _parse_enabled(d.pop("enabled"))

        def _parse_enabled_by(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        enabled_by = _parse_enabled_by(d.pop("enabledBy"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        public_key = cls(
            id=id,
            user_id=user_id,
            hash_=hash_,
            data=data,
            is_root_key=is_root_key,
            enabled=enabled,
            enabled_by=enabled_by,
            created_at=created_at,
            updated_at=updated_at,
        )

        return public_key



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.inode_members import InodeMembers


T = TypeVar("T", bound="ListResponse")


@_attrs_define
class ListResponse:
    """
    Attributes:
        parents (List['InodeMembers']): The list of parent directories
        children (List['InodeMembers']): The list of inodes in the directory
    """

    parents: List["InodeMembers"]
    children: List["InodeMembers"]

    def to_dict(self) -> Dict[str, Any]:
        parents = []
        for parents_item_data in self.parents:
            parents_item = parents_item_data.to_dict()
            parents.append(parents_item)

        children = []
        for children_item_data in self.children:
            children_item = children_item_data.to_dict()
            children.append(children_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "parents": parents,
                "children": children,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.inode_members import InodeMembers

        d = src_dict.copy()
        parents = []
        _parents = d.pop("parents")
        for parents_item_data in _parents:
            parents_item = InodeMembers.from_dict(parents_item_data)

            parents.append(parents_item)

        children = []
        _children = d.pop("children")
        for children_item_data in _children:
            children_item = InodeMembers.from_dict(children_item_data)

            children.append(children_item)

        list_response = cls(
            parents=parents,
            children=children,
        )

        return list_response



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.dabih_info_branding import DabihInfoBranding


T = TypeVar("T", bound="DabihInfo")


@_attrs_define
class DabihInfo:
    """
    Attributes:
        version (str):
        branding (DabihInfoBranding):
    """

    version: str
    branding: "DabihInfoBranding"

    def to_dict(self) -> Dict[str, Any]:
        version = self.version

        branding = self.branding.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "version": version,
                "branding": branding,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dabih_info_branding import DabihInfoBranding

        d = src_dict.copy()
        version = d.pop("version")

        branding = DabihInfoBranding.from_dict(d.pop("branding"))

        dabih_info = cls(
            version=version,
            branding=branding,
        )

        return dabih_info



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="UploadStartBody")


@_attrs_define
class UploadStartBody:
    """
    Attributes:
        file_name (str): The name of the file to upload
        directory (Union[Unset, str]): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        file_path (Union[Unset, str]): The original path of the file
        size (Union[Unset, int]): The size of the file in bytes
        tag (Union[Unset, str]): A custom searchable tag for the file
    """

    file_name: str
    directory: Union[Unset, str] = UNSET
    file_path: Union[Unset, str] = UNSET
    size: Union[Unset, int] = UNSET
    tag: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        file_name = self.file_name

        directory = self.directory

        file_path = self.file_path

        size = self.size

        tag = self.tag

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "fileName": file_name,
            }
        )
        if directory is not UNSET:
            field_dict["directory"] = directory
        if file_path is not UNSET:
            field_dict["filePath"] = file_path
        if size is not UNSET:
            field_dict["size"] = size
        if tag is not UNSET:
            field_dict["tag"] = tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        file_name = d.pop("fileName")

        directory = d.pop("directory", UNSET)

        file_path = d.pop("filePath", UNSET)

        size = d.pop("size", UNSET)

        tag = d.pop("tag", UNSET)

        upload_start_body = cls(
            file_name=file_name,
            directory=directory,
            file_path=file_path,
            size=size,
            tag=tag,
        )

        return upload_start_body



"""Contains all the data models used in inputs/outputs"""

from .add_directory_body import AddDirectoryBody
from .chunk import Chunk
from .chunk_data import ChunkData
from .chunk_upload_body import ChunkUploadBody
from .crypto_json_web_key import CryptoJsonWebKey
from .dabih_info import DabihInfo
from .dabih_info_branding import DabihInfoBranding
from .dabih_info_branding_admin import DabihInfoBrandingAdmin
from .dabih_info_branding_contact import DabihInfoBrandingContact
from .dabih_info_branding_department import DabihInfoBrandingDepartment
from .dabih_info_branding_organization import DabihInfoBrandingOrganization
from .decrypt_dataset_body import DecryptDatasetBody
from .directory import Directory
from .file_data import FileData
from .file_decryption_key import FileDecryptionKey
from .find_user_body import FindUserBody
from .healthy_response_200 import HealthyResponse200
from .inode import Inode
from .inode_members import InodeMembers
from .inode_search_body import InodeSearchBody
from .inode_search_results import InodeSearchResults
from .inode_tree import InodeTree
from .inode_type import InodeType
from .job import Job
from .job_status import JobStatus
from .key import Key
from .key_add_body import KeyAddBody
from .key_enable_body import KeyEnableBody
from .key_remove_body import KeyRemoveBody
from .list_response import ListResponse
from .member import Member
from .member_add_body import MemberAddBody
from .move_inode_body import MoveInodeBody
from .permission import Permission
from .public_key import PublicKey
from .remove_token_body import RemoveTokenBody
from .remove_user_body import RemoveUserBody
from .search_fs_response_200 import SearchFsResponse200
from .token import Token
from .token_add_body import TokenAddBody
from .token_response import TokenResponse
from .upload_start_body import UploadStartBody
from .user import User
from .user_add_body import UserAddBody
from .user_response import UserResponse

__all__ = (
    "AddDirectoryBody",
    "Chunk",
    "ChunkData",
    "ChunkUploadBody",
    "CryptoJsonWebKey",
    "DabihInfo",
    "DabihInfoBranding",
    "DabihInfoBrandingAdmin",
    "DabihInfoBrandingContact",
    "DabihInfoBrandingDepartment",
    "DabihInfoBrandingOrganization",
    "DecryptDatasetBody",
    "Directory",
    "FileData",
    "FileDecryptionKey",
    "FindUserBody",
    "HealthyResponse200",
    "Inode",
    "InodeMembers",
    "InodeSearchBody",
    "InodeSearchResults",
    "InodeTree",
    "InodeType",
    "Job",
    "JobStatus",
    "Key",
    "KeyAddBody",
    "KeyEnableBody",
    "KeyRemoveBody",
    "ListResponse",
    "Member",
    "MemberAddBody",
    "MoveInodeBody",
    "Permission",
    "PublicKey",
    "RemoveTokenBody",
    "RemoveUserBody",
    "SearchFsResponse200",
    "Token",
    "TokenAddBody",
    "TokenResponse",
    "UploadStartBody",
    "User",
    "UserAddBody",
    "UserResponse",
)



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.crypto_json_web_key import CryptoJsonWebKey


T = TypeVar("T", bound="KeyAddBody")


@_attrs_define
class KeyAddBody:
    """
    Attributes:
        sub (str): The user the key should belong to
        data (CryptoJsonWebKey):
        is_root_key (bool): If true the key is a root key, used to decrypt all datasets
    """

    sub: str
    data: "CryptoJsonWebKey"
    is_root_key: bool

    def to_dict(self) -> Dict[str, Any]:
        sub = self.sub

        data = self.data.to_dict()

        is_root_key = self.is_root_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "sub": sub,
                "data": data,
                "isRootKey": is_root_key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.crypto_json_web_key import CryptoJsonWebKey

        d = src_dict.copy()
        sub = d.pop("sub")

        data = CryptoJsonWebKey.from_dict(d.pop("data"))

        is_root_key = d.pop("isRootKey")

        key_add_body = cls(
            sub=sub,
            data=data,
            is_root_key=is_root_key,
        )

        return key_add_body



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.inode_type import InodeType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_data import FileData
    from ..models.member import Member


T = TypeVar("T", bound="InodeMembers")


@_attrs_define
class InodeMembers:
    """
    Attributes:
        id (Any): The database id of the inode
        mnemonic (str):
        type (InodeType): InodeType is used to represent the type of an Inode.
            FILE: a file
            DIRECTORY: a directory
            UPLOAD: a file that is being uploaded
            TRASH: the special directory that holds deleted files
            ROOT: the global root directory
            HOME: the user's home directory
            USERS: the directory that holds all user directories
        name (str):
        tag (Union[None, str]):
        parent_id (Any):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        members (List['Member']):
        data (Union['FileData', None, Unset]):
    """

    id: Any
    mnemonic: str
    type: InodeType
    name: str
    tag: Union[None, str]
    parent_id: Any
    created_at: datetime.datetime
    updated_at: datetime.datetime
    members: List["Member"]
    data: Union["FileData", None, Unset] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.file_data import FileData

        id = self.id

        mnemonic = self.mnemonic

        type = self.type.value

        name = self.name

        tag: Union[None, str]
        tag = self.tag

        parent_id = self.parent_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        members = []
        for members_item_data in self.members:
            members_item = members_item_data.to_dict()
            members.append(members_item)

        data: Union[Dict[str, Any], None, Unset]
        if isinstance(self.data, Unset):
            data = UNSET
        elif isinstance(self.data, FileData):
            data = self.data.to_dict()
        else:
            data = self.data

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "mnemonic": mnemonic,
                "type": type,
                "name": name,
                "tag": tag,
                "parentId": parent_id,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "members": members,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.file_data import FileData
        from ..models.member import Member

        d = src_dict.copy()
        id = d.pop("id")

        mnemonic = d.pop("mnemonic")

        type = InodeType(d.pop("type"))

        name = d.pop("name")

        def _parse_tag(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        tag = _parse_tag(d.pop("tag"))

        parent_id = d.pop("parentId")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        members = []
        _members = d.pop("members")
        for members_item_data in _members:
            members_item = Member.from_dict(members_item_data)

            members.append(members_item)

        def _parse_data(data: object) -> Union["FileData", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                data_type_1 = FileData.from_dict(data)

                return data_type_1
            except:  # noqa: E722
                pass
            return cast(Union["FileData", None, Unset], data)

        data = _parse_data(d.pop("data", UNSET))

        inode_members = cls(
            id=id,
            mnemonic=mnemonic,
            type=type,
            name=name,
            tag=tag,
            parent_id=parent_id,
            created_at=created_at,
            updated_at=updated_at,
            members=members,
            data=data,
        )

        inode_members.additional_properties = d
        return inode_members

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import click
import sys
from .config import *
from .helpers import *
from dabih.logger import setup_logger, dbg, error

@click.group()
@click.option("-v", "--verbose", is_flag=True, help="Enable verbose output")
@click.pass_context
def main(ctx, verbose):
    setup_logger(verbose)
    client, pem_files = get_client()
    if not healthy_func(client):
        error("Dabih server not healthy")
        dbg(f"client: {client}")
        sys.exit(0)
    else:
        dbg("Dabi server healthy")

    key_files, *_ = get_user_info(client)
    if pem_files:
        pem_files = get_user_key_files(key_files, pem_files)
    ctx.obj = {}
    ctx.obj["client"] = client
    ctx.obj["pem_files"] = pem_files


@click.command(
    short_help="Check if current token is valid", help="This command will check if token is valid"
)
@click.pass_context
def token_val(ctx):
    client = ctx.obj["client"]
    answer = check_token_validity(client)
    if answer:
        print("Token is valid")
    else:
        print("Token is not valid")


@click.command(
    short_help="Get current token scope info",
    help="This command will return user and role of the token",
)
@click.pass_context
def token_info(ctx):
    client = ctx.obj["client"]
    get_token_info(client)

@click.command(
    short_help="Check user credentials including token and available key files", help="Check token and existing key files"
)
@click.pass_context
def check(ctx):
    client = ctx.obj["client"]
    pem_files = ctx.obj["pem_files"]
    check_user_credentials(client, pem_files)


@click.command(
    short_help="List all files in user's home",
    help="see all files and folders in home",
)
@click.pass_context
def list_home(ctx):
    client = ctx.obj["client"]
    list_home_func(client)

@click.command(
    short_help="List all files in specific folder",
    help="enter mnemonic e.g. dabih list-files rufus_shane to see file info or files under directory",
)
@click.pass_context
@click.argument("mnemonic")
def list_files(ctx, mnemonic):
    client = ctx.obj["client"]
    answer = list_files_func(mnemonic, client).content
    print_json(answer)


@click.command(
    short_help="Upload a file", help="This command will upload a file to dabih server"
)
@click.pass_context
@click.argument("file_path")
@click.argument("target_directory", required=False, default=None)
def upload(ctx, file_path, target_directory):  # potential namespace conflict?
    client = ctx.obj["client"]
    try:
        upload_func(file_path, client, target_directory)
    except FileNotFoundError as e:
        error(f"File at path: {file_path} not found. Please check the path.")
        dbg(f"Error: {e}")
        sys.exit(0)
    except Exception as e:
        error("Unexpected error")
        dbg(f"Error: {e}")

@click.command(
    short_help="Download a file",
    help="This command will download a file to dabih server",
)
@click.pass_context
@click.argument("mnemonic")
def download(ctx, mnemonic):
    client = ctx.obj["client"]
    if not ctx.obj["pem_files"]:
        print("No valid key files found; you need a private key to download files")
        return None
    download_func(mnemonic, client, ctx.obj["pem_files"])


@click.command(
    short_help="Search for a file or folder", help="This command will return all search results"
)
@click.pass_context
@click.argument("query")
def search(ctx, query):
    client = ctx.obj["client"]
    search_func(client, query)


main.add_command(token_info)
main.add_command(token_val)
main.add_command(check)
main.add_command(list_home)
main.add_command(list_files)
main.add_command(upload)
main.add_command(download)
main.add_command(search)


if __name__ == "__main__":
    main()




import datetime
from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

T = TypeVar("T", bound="Key")


@_attrs_define
class Key:
    """
    Attributes:
        id (Any): The database id of the key
        inode_id (Any): The inode id the key belongs to
        hash_ (str):
        key (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    id: Any
    inode_id: Any
    hash_: str
    key: str
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        inode_id = self.inode_id

        hash_ = self.hash_

        key = self.key

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "inodeId": inode_id,
                "hash": hash_,
                "key": key,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        inode_id = d.pop("inodeId")

        hash_ = d.pop("hash")

        key = d.pop("key")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        key = cls(
            id=id,
            inode_id=inode_id,
            hash_=hash_,
            key=key,
            created_at=created_at,
            updated_at=updated_at,
        )

        return key



from dabih.dabih_client.dabih_api_client.api.util import (
    healthy,
    info
)
from ..logger import dbg, error, warn
import json
import sys
import httpx

__all__ = ["healthy_func", "check_status"]

def healthy_func(client):

    try: 
        answer = healthy.sync_detailed(client=client)
    except httpx.ConnectError:
        error(f"Connection error: Please check the URL in config file or whether the server is running. \n Required format of base_url: http://<ip>:<port>")
        sys.exit(0)
        
    if not check_status(answer):
        return False
    else:
        return True

def check_status(answer, context = None):
    dbg("Checking Server Response...")

    if answer.status_code == 401:
        error("Token is unauthorized or expired")
        dbg(f"Server Response: {answer.content}")
        sys.exit(0)

    elif answer.status_code == 403:
        error("Not authorized to access this dabih folder/file")
        dbg(f"Server Response: {answer.content}")
        sys.exit(0)

    elif answer.status_code == 404:
        message = json.loads(answer.content)
        if context == "file_info":
            error(f"Requested dabih file/folder not found: {message['message']}. Only files, not folders, can be downloaded.")
        else:
            error(f"Requested dabih file/folder not found: {message['message']}.")
            warn("Dabih Files/Folders should always be refered to by their mnemonic. \nYou can use dabih search to check the mnemonic")
        dbg(f"Server Response: {answer.content}")
        sys.exit(0)

    elif answer.status_code == 500:
        try:
            response = json.loads(answer.content)
            dbg(f"Server Message: {response['message']}")
            if response["message"] == "jwt malformed":
                error("Token is malformed. Please check your token in the config file")
                sys.exit(0)
            else:
                error(f"Error with server message {response['message']}")
                sys.exit(0)
        except:
            error("Server Side Issue")
            dbg(f"Server Response: {answer.content}")
            return False

    elif answer.status_code == 200:
        dbg(f"Server Status Code: {answer.status_code}")
        dbg(f"Server Response: {answer.content}")
        return True
    
    elif answer.status_code == 201:
        dbg(f"Server Status Code: {answer.status_code}")
        dbg(f"Server Response: {answer.content}")
        return True


from dabih.dabih_client.dabih_api_client.api.user import me
from .json import decode_json
from .crypto import hash
from .token import get_token_info
from .util import healthy_func, check_status
from ..logger import dbg, warn, log, error
from cryptography.hazmat.primitives import serialization
import httpx
import sys

__all__=["get_user_info", "get_user_key_files", "check_user_credentials"]

def get_user_info(client):
    dbg("getting user info...")
    try:
        answer = me.sync_detailed(client = client)
    except httpx.ConnectError:
        error(f"Connection error: Please check the URL in config file or whether the server is running. \n Required format of base_url: http://<ip>:<port>")
        sys.exit(0)
    except:
        error(f"Connection error: Please check the URL in config file or whether the server is running. \n Required format of base_url: http://<ip>:<port>")
        sys.exit(0)
    check_status(answer)

    data = decode_json(answer.content)
    key_list = data["keys"]
    id = data["id"]
    name = data["name"]
    dbg(f"User ID: {id}, Name: {name}, Key List: {key_list}")
    return key_list, name

def get_user_key_files(key_list, pem_files):
    dbg("getting user specific key files...")
    dbg(f"Key List: {key_list}\nPem Files: {pem_files}")
    for pem_file in pem_files:
        pem_file_obj = open(pem_file, "rb")
        try:
            private_key = serialization.load_pem_private_key(
                pem_file_obj.read(),
                password=None 
            )
            pem_hash = hash.get_key_hash(private_key)
        except ValueError:
            warn(f"Error loading key from PEM file {pem_file}\nThis key file has an invalid format. Using another key if available.")
            pem_file_obj.close()
            pem_files.remove(pem_file)
            continue
        finally:
            pem_file_obj.close()

        if pem_hash not in [key["hash"] for key in key_list]:
            pem_files.remove(pem_file)

    return pem_files

def check_user_credentials(client, pem_files):
    
    *_, name = get_user_info(client)
    log(f"User name: {name}")

    if not pem_files:
        error(f"No valid key files for user with name {name} found")
    else:
        log(f"Found {len(pem_files)} valid key files for user with name {name}: {pem_files}")
    
    get_token_info(client)


from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="CryptoJsonWebKey")


@_attrs_define
class CryptoJsonWebKey:
    """
    Attributes:
        crv (Union[Unset, str]):
        d (Union[Unset, str]):
        dp (Union[Unset, str]):
        dq (Union[Unset, str]):
        e (Union[Unset, str]):
        k (Union[Unset, str]):
        kty (Union[Unset, str]):
        n (Union[Unset, str]):
        p (Union[Unset, str]):
        q (Union[Unset, str]):
        qi (Union[Unset, str]):
        x (Union[Unset, str]):
        y (Union[Unset, str]):
    """

    crv: Union[Unset, str] = UNSET
    d: Union[Unset, str] = UNSET
    dp: Union[Unset, str] = UNSET
    dq: Union[Unset, str] = UNSET
    e: Union[Unset, str] = UNSET
    k: Union[Unset, str] = UNSET
    kty: Union[Unset, str] = UNSET
    n: Union[Unset, str] = UNSET
    p: Union[Unset, str] = UNSET
    q: Union[Unset, str] = UNSET
    qi: Union[Unset, str] = UNSET
    x: Union[Unset, str] = UNSET
    y: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        crv = self.crv

        d = self.d

        dp = self.dp

        dq = self.dq

        e = self.e

        k = self.k

        kty = self.kty

        n = self.n

        p = self.p

        q = self.q

        qi = self.qi

        x = self.x

        y = self.y

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if crv is not UNSET:
            field_dict["crv"] = crv
        if d is not UNSET:
            field_dict["d"] = d
        if dp is not UNSET:
            field_dict["dp"] = dp
        if dq is not UNSET:
            field_dict["dq"] = dq
        if e is not UNSET:
            field_dict["e"] = e
        if k is not UNSET:
            field_dict["k"] = k
        if kty is not UNSET:
            field_dict["kty"] = kty
        if n is not UNSET:
            field_dict["n"] = n
        if p is not UNSET:
            field_dict["p"] = p
        if q is not UNSET:
            field_dict["q"] = q
        if qi is not UNSET:
            field_dict["qi"] = qi
        if x is not UNSET:
            field_dict["x"] = x
        if y is not UNSET:
            field_dict["y"] = y

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        crv = d.pop("crv", UNSET)

        d = d.pop("d", UNSET)

        dp = d.pop("dp", UNSET)

        dq = d.pop("dq", UNSET)

        e = d.pop("e", UNSET)

        k = d.pop("k", UNSET)

        kty = d.pop("kty", UNSET)

        n = d.pop("n", UNSET)

        p = d.pop("p", UNSET)

        q = d.pop("q", UNSET)

        qi = d.pop("qi", UNSET)

        x = d.pop("x", UNSET)

        y = d.pop("y", UNSET)

        crypto_json_web_key = cls(
            crv=crv,
            d=d,
            dp=dp,
            dq=dq,
            e=e,
            k=k,
            kty=kty,
            n=n,
            p=p,
            q=q,
            qi=qi,
            x=x,
            y=y,
        )

        crypto_json_web_key.additional_properties = d
        return crypto_json_web_key

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_decryption_key import FileDecryptionKey


T = TypeVar("T", bound="MoveInodeBody")


@_attrs_define
class MoveInodeBody:
    """
    Attributes:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        parent (Union[None, Unset, str]): Optional: The mnemonic of the new parent directory
        keys (Union[Unset, List['FileDecryptionKey']]): The list of AES-256 keys required to decrypt all child datasets
        name (Union[Unset, str]): Optional: The new name of the inode
        tag (Union[Unset, str]): Optional: The new tag of the inode
    """

    mnemonic: str
    parent: Union[None, Unset, str] = UNSET
    keys: Union[Unset, List["FileDecryptionKey"]] = UNSET
    name: Union[Unset, str] = UNSET
    tag: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        mnemonic = self.mnemonic

        parent: Union[None, Unset, str]
        if isinstance(self.parent, Unset):
            parent = UNSET
        else:
            parent = self.parent

        keys: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.keys, Unset):
            keys = []
            for keys_item_data in self.keys:
                keys_item = keys_item_data.to_dict()
                keys.append(keys_item)

        name = self.name

        tag = self.tag

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "mnemonic": mnemonic,
            }
        )
        if parent is not UNSET:
            field_dict["parent"] = parent
        if keys is not UNSET:
            field_dict["keys"] = keys
        if name is not UNSET:
            field_dict["name"] = name
        if tag is not UNSET:
            field_dict["tag"] = tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.file_decryption_key import FileDecryptionKey

        d = src_dict.copy()
        mnemonic = d.pop("mnemonic")

        def _parse_parent(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        parent = _parse_parent(d.pop("parent", UNSET))

        keys = []
        _keys = d.pop("keys", UNSET)
        for keys_item_data in _keys or []:
            keys_item = FileDecryptionKey.from_dict(keys_item_data)

            keys.append(keys_item)

        name = d.pop("name", UNSET)

        tag = d.pop("tag", UNSET)

        move_inode_body = cls(
            mnemonic=mnemonic,
            parent=parent,
            keys=keys,
            name=name,
            tag=tag,
        )

        return move_inode_body



from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.chunk import Chunk
from ...models.chunk_upload_body import ChunkUploadBody
from ...types import Response


def _get_kwargs(
    mnemonic: str,
    *,
    body: ChunkUploadBody,
    content_range: str,
    digest: str,
) -> Dict[str, Any]:
    headers: Dict[str, Any] = {}
    headers["content-range"] = content_range

    headers["digest"] = digest

    _kwargs: Dict[str, Any] = {
        "method": "put",
        "url": f"/upload/{mnemonic}/chunk",
    }

    _body = body.to_multipart()

    _kwargs["files"] = _body

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Optional[Chunk]:
    if response.status_code == 201:
        response_201 = Chunk.from_dict(response.json())

        return response_201
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client: Union[AuthenticatedClient, Client], response: httpx.Response) -> Response[Chunk]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    mnemonic: str,
    *,
    client: AuthenticatedClient,
    body: ChunkUploadBody,
    content_range: str,
    digest: str,
) -> Response[Chunk]:
    """
    Args:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        content_range (str):
        digest (str):
        body (ChunkUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Chunk]
    """

    kwargs = _get_kwargs(
        mnemonic=mnemonic,
        body=body,
        content_range=content_range,
        digest=digest,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    mnemonic: str,
    *,
    client: AuthenticatedClient,
    body: ChunkUploadBody,
    content_range: str,
    digest: str,
) -> Optional[Chunk]:
    """
    Args:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        content_range (str):
        digest (str):
        body (ChunkUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Chunk
    """

    return sync_detailed(
        mnemonic=mnemonic,
        client=client,
        body=body,
        content_range=content_range,
        digest=digest,
    ).parsed


async def asyncio_detailed(
    mnemonic: str,
    *,
    client: AuthenticatedClient,
    body: ChunkUploadBody,
    content_range: str,
    digest: str,
) -> Response[Chunk]:
    """
    Args:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        content_range (str):
        digest (str):
        body (ChunkUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Chunk]
    """

    kwargs = _get_kwargs(
        mnemonic=mnemonic,
        body=body,
        content_range=content_range,
        digest=digest,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    mnemonic: str,
    *,
    client: AuthenticatedClient,
    body: ChunkUploadBody,
    content_range: str,
    digest: str,
) -> Optional[Chunk]:
    """
    Args:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        content_range (str):
        digest (str):
        body (ChunkUploadBody):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Chunk
    """

    return (
        await asyncio_detailed(
            mnemonic=mnemonic,
            client=client,
            body=body,
            content_range=content_range,
            digest=digest,
        )
    ).parsed



from dabih.dabih_client.dabih_api_client.api.download import download_chunk
from .filesystem import get_file_info
from .crypto import b64, decrypt, hash
from ..logger import dbg, warn, log, error
from .util import check_status
from cryptography.hazmat.primitives import serialization
import sys

__all__ = ["find_key", "download_func"]


def find_key(key_list, pem_files):
    dbg("Finding private key...")
    dbg(f"key_list: {key_list}")
    for key in key_list:
        dbg(f"pem_list: {pem_files}")
        for pem_file in pem_files:
            try:
                with open(pem_file, "rb") as pem_file:
                    private_key = serialization.load_pem_private_key(
                        pem_file.read(),
                        password=None 
                    )
                    pem_hash = hash.get_key_hash(private_key)
            except (ValueError, TypeError) as e:
                warn(f"Error loading this PEM file {pem_file}. Using another key if available...")
                continue
                
            if key["hash"] == pem_hash:
                log("Found valid key")
                return private_key, key
            else:
                dbg(f"pem_hash: {pem_hash}")
                dbg(f"key_hash: {key['hash']}")

    warn("No valid key found")
    return None


def download_func(mnemonic, client, pem_files):
    log(f"Starting download for mnemonic: {mnemonic}")
    file, uid, key_list, size = get_file_info(mnemonic, client)
    if not pem_files:
        error("No valid key files found. You need your private key to download files. \nDownload aborted.")
        sys.exit(0)
    private_key, public_key = find_key(key_list, pem_files)
    
    if not private_key:
        error("No private key  found for requested file. Please check your key files.")
        sys.exit(0)

    encrypted_aes_key = public_key["key"]
    aes_key = decrypt.decrypt_aes_key(encrypted_aes_key, private_key)

    start = 0
    size = float(size)

    with open(file["fileName"], "wb") as f:
        log("Downloading... 0%")
        for chunk in file["chunks"]:

            chunk_hash = chunk["hash"]
            chunk_response = download_chunk.sync_detailed(uid=uid, hash_=chunk_hash, client=client)
            check_status(chunk_response)
            encrypted_chunk = chunk_response.content
            n = len(encrypted_chunk)

            iv = b64.decode(chunk["iv"])
            decrypted_chunk = decrypt.decrypt_file_chunk(encrypted_chunk, aes_key, iv)
        
            f.write(decrypted_chunk)

            last_percent = (start * 100) // size
            start += n
            percent = (start * 100) // size
            if percent != last_percent:
                print(f"{percent}%", end=" \n")
                sys.stdout.flush()

    log("Download finished.")


from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="KeyRemoveBody")


@_attrs_define
class KeyRemoveBody:
    """
    Attributes:
        sub (str): The user the key belongs to
        hash_ (str): The hash of the key
    """

    sub: str
    hash_: str

    def to_dict(self) -> Dict[str, Any]:
        sub = self.sub

        hash_ = self.hash_

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "sub": sub,
                "hash": hash_,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sub = d.pop("sub")

        hash_ = d.pop("hash")

        key_remove_body = cls(
            sub=sub,
            hash_=hash_,
        )

        return key_remove_body



import datetime
from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define
from dateutil.parser import isoparse

from ..models.permission import Permission

T = TypeVar("T", bound="Member")


@_attrs_define
class Member:
    """
    Attributes:
        id (Any): The database id of the member
        sub (str):
        inode_id (Any): The database id of the inode
        permission (Permission): The Permission type is used to represent the permissions a user has on a Inode.
            NONE: No permissions
            READ: user may only read the file or directory
            WRITE: user can read, share, edit, and delete the file or directory
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
    """

    id: Any
    sub: str
    inode_id: Any
    permission: Permission
    created_at: datetime.datetime
    updated_at: datetime.datetime

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        sub = self.sub

        inode_id = self.inode_id

        permission = self.permission.value

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "sub": sub,
                "inodeId": inode_id,
                "permission": permission,
                "createdAt": created_at,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        sub = d.pop("sub")

        inode_id = d.pop("inodeId")

        permission = Permission(d.pop("permission"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        member = cls(
            id=id,
            sub=sub,
            inode_id=inode_id,
            permission=permission,
            created_at=created_at,
            updated_at=updated_at,
        )

        return member



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="RemoveUserBody")


@_attrs_define
class RemoveUserBody:
    """
    Attributes:
        sub (str):
    """

    sub: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sub = self.sub

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sub": sub,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sub = d.pop("sub")

        remove_user_body = cls(
            sub=sub,
        )

        remove_user_body.additional_properties = d
        return remove_user_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import base64

def decode(iv):
    iv += '=' * (-len(iv) % 4)
    iv = base64.urlsafe_b64decode(iv)
    return iv

def encode(iv):
    iv = base64.urlsafe_b64encode(iv).rstrip(b'=').decode('utf-8')
    return iv



from typing import TYPE_CHECKING, Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.crypto_json_web_key import CryptoJsonWebKey


T = TypeVar("T", bound="UserAddBody")


@_attrs_define
class UserAddBody:
    """
    Attributes:
        name (str): The name of the user
        email (str): The email of the user
        key (CryptoJsonWebKey):
        sub (Union[Unset, str]): The unique user sub
            if undefined the sub from the auth token will be used
        is_root_key (Union[Unset, bool]): If true the key is a root key, used to decrypt all datasets
    """

    name: str
    email: str
    key: "CryptoJsonWebKey"
    sub: Union[Unset, str] = UNSET
    is_root_key: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        email = self.email

        key = self.key.to_dict()

        sub = self.sub

        is_root_key = self.is_root_key

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "email": email,
                "key": key,
            }
        )
        if sub is not UNSET:
            field_dict["sub"] = sub
        if is_root_key is not UNSET:
            field_dict["isRootKey"] = is_root_key

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.crypto_json_web_key import CryptoJsonWebKey

        d = src_dict.copy()
        name = d.pop("name")

        email = d.pop("email")

        key = CryptoJsonWebKey.from_dict(d.pop("key"))

        sub = d.pop("sub", UNSET)

        is_root_key = d.pop("isRootKey", UNSET)

        user_add_body = cls(
            name=name,
            email=email,
            key=key,
            sub=sub,
            is_root_key=is_root_key,
        )

        return user_add_body



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

if TYPE_CHECKING:
    from ..models.chunk import Chunk


T = TypeVar("T", bound="ChunkData")


@_attrs_define
class ChunkData:
    """
    Attributes:
        id (Any): The database id of the file data
        uid (str):
        created_by (str):
        file_name (str):
        file_path (Union[None, str]):
        hash_ (Union[None, str]):
        size (Any): The size of the file in bytes
        key_hash (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        chunks (List['Chunk']):
    """

    id: Any
    uid: str
    created_by: str
    file_name: str
    file_path: Union[None, str]
    hash_: Union[None, str]
    size: Any
    key_hash: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    chunks: List["Chunk"]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        uid = self.uid

        created_by = self.created_by

        file_name = self.file_name

        file_path: Union[None, str]
        file_path = self.file_path

        hash_: Union[None, str]
        hash_ = self.hash_

        size = self.size

        key_hash = self.key_hash

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        chunks = []
        for chunks_item_data in self.chunks:
            chunks_item = chunks_item_data.to_dict()
            chunks.append(chunks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "uid": uid,
                "createdBy": created_by,
                "fileName": file_name,
                "filePath": file_path,
                "hash": hash_,
                "size": size,
                "keyHash": key_hash,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "chunks": chunks,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chunk import Chunk

        d = src_dict.copy()
        id = d.pop("id")

        uid = d.pop("uid")

        created_by = d.pop("createdBy")

        file_name = d.pop("fileName")

        def _parse_file_path(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        file_path = _parse_file_path(d.pop("filePath"))

        def _parse_hash_(data: object) -> Union[None, str]:
            if data is None:
                return data
            return cast(Union[None, str], data)

        hash_ = _parse_hash_(d.pop("hash"))

        size = d.pop("size")

        key_hash = d.pop("keyHash")

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        chunks = []
        _chunks = d.pop("chunks")
        for chunks_item_data in _chunks:
            chunks_item = Chunk.from_dict(chunks_item_data)

            chunks.append(chunks_item)

        chunk_data = cls(
            id=id,
            uid=uid,
            created_by=created_by,
            file_name=file_name,
            file_path=file_path,
            hash_=hash_,
            size=size,
            key_hash=key_hash,
            created_at=created_at,
            updated_at=updated_at,
            chunks=chunks,
        )

        chunk_data.additional_properties = d
        return chunk_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



# decryption
from cryptography.hazmat.primitives import hashes, ciphers, serialization
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers import algorithms, modes
from cryptography.hazmat.backends import default_backend

from .b64 import decode as b64_decode

def decrypt_aes_key(encrypted_aes_key, private_key):
    encrypted_aes_key_bytes = b64_decode(encrypted_aes_key)
    decrypted_aes_key = private_key.decrypt(
        encrypted_aes_key_bytes,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )
    return decrypted_aes_key


def decrypt_file_chunk(chunk_data, aes_key, iv):
    cipher = ciphers.Cipher(
        algorithms.AES(aes_key), modes.CBC(iv), backend=default_backend()
    )
    decryptor = cipher.decryptor()
    decrypted_data = decryptor.update(chunk_data) + decryptor.finalize()
    return decrypted_data



from dabih.dabih_client.dabih_api_client.api.upload import (
    start_upload,
    chunk_upload,
    finish_upload,
    cancel_upload,
    unfinished_uploads,
)
from dabih.dabih_client.dabih_api_client.types import File
from dabih.dabih_client.dabih_api_client.models.upload_start_body import UploadStartBody
from dabih.dabih_client.dabih_api_client.models.chunk_upload_body import ChunkUploadBody

from .json import decode_json
from .crypto import hash
from .util import check_status
from ..logger import dbg, warn, log, error

from io import BytesIO
from pathlib import Path
import sys


__all__ = ["upload_func"]

def start_upload_func(file_name, size, client, directory = None):
    
    if directory is None: 
        body = UploadStartBody(file_name=file_name, size=size)
    else:
        body = UploadStartBody(file_name=file_name, size=size, directory=directory)
    dbg(f"Upload Body: {body}")

    answer = start_upload.sync_detailed(client=client, body=body)
    check_status(answer)
    return answer

def upload_chunk(mnemonic, chunk_data, start, end, total_size, client):
    
    
    log(f"Preparing to upload chunk from {start} to {end}")

    file_instance = File(payload=BytesIO(chunk_data), file_name="chunk", mime_type=None)
    dbg(f"File Instance: {file_instance}")
    body = ChunkUploadBody(chunk=file_instance)
    dbg(f"Chunk Upload Body: {body}")

    content_range = f"bytes {start}-{end}/{total_size}"
    digest = f"sha-256={hash.get_chunk_hash(chunk_data)}"

    response = chunk_upload.sync_detailed(
        mnemonic=mnemonic,
        client=client,
        body=body,
        content_range=content_range,
        digest=digest,
    )
    check_status(response)

    return response


def finish_upload_func(mnemonic, client):

    answer = finish_upload.sync_detailed(client=client, mnemonic=mnemonic)
    check_status(answer)
    return answer


def upload_func(filepath, client, target_directory=None):

    total_size = Path(filepath).stat().st_size
    chunk_size = 2 * 1024 * 1024
    start = 0

    start_upload_answer = start_upload_func(filepath, total_size, client, target_directory)
    mnemonic = decode_json(start_upload_answer.content)["mnemonic"]

    log(f"Upload started with mnemonic: {mnemonic}")
    hashes = []

    with open(filepath, "rb") as f:
        log("Uploading... 0%")
        while True:
            chunk_data = f.read(chunk_size)
            n = len(chunk_data)
            if n == 0:  # break while loop if no data left
                break
            end = start + n - 1
            chunk_answer = upload_chunk(
                mnemonic, chunk_data, start, end, total_size, client
            )
            last_percent = (start * 100) // total_size
            start += n
            percent = (start * 100) // total_size
            if percent != last_percent:
                print(f"{percent}%", end=" ")
                sys.stdout.flush()
            chunk_hash = decode_json(chunk_answer.content)["hash"]
            hashes.append(chunk_hash)
            dbg(f"hashlist from server: {hashes}")

    result_answer = finish_upload_func(mnemonic, client)
    result_hash = decode_json(result_answer.content)["data"]["hash"]

    full_hash = hash.get_full_chunk_hash(hashes)
    if result_hash != full_hash:
        raise Exception(
            f"Upload failed: hashes do not match: {result_hash} != {full_hash}"
        )
    log("\nUpload finished.")


from typing import Any, Dict, List, Type, TypeVar, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="User")


@_attrs_define
class User:
    """User is the type that represents a user in the system.

    Attributes:
        sub (str): The subject of the user, a unique identifier Example: mhuttner.
        scopes (List[str]): The scopes the user has Example: ['dabih:api'].
        is_admin (bool): Does the user have the dabih:admin scope
    """

    sub: str
    scopes: List[str]
    is_admin: bool

    def to_dict(self) -> Dict[str, Any]:
        sub = self.sub

        scopes = self.scopes

        is_admin = self.is_admin

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "sub": sub,
                "scopes": scopes,
                "isAdmin": is_admin,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        sub = d.pop("sub")

        scopes = cast(List[str], d.pop("scopes"))

        is_admin = d.pop("isAdmin")

        user = cls(
            sub=sub,
            scopes=scopes,
            is_admin=is_admin,
        )

        return user



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="TokenResponse")


@_attrs_define
class TokenResponse:
    """
    Attributes:
        id (Any): The id of the token
        value (str):
        sub (str):
        scope (str):
        exp (Union[None, datetime.datetime]):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        expired (Union[bool, str]): false if the token has not expired,
            otherwise a string describing how long ago the token expired
        scopes (List[str]): The array of scopes the token has
    """

    id: Any
    value: str
    sub: str
    scope: str
    exp: Union[None, datetime.datetime]
    created_at: datetime.datetime
    updated_at: datetime.datetime
    expired: Union[bool, str]
    scopes: List[str]
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        value = self.value

        sub = self.sub

        scope = self.scope

        exp: Union[None, str]
        if isinstance(self.exp, datetime.datetime):
            exp = self.exp.isoformat()
        else:
            exp = self.exp

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        expired: Union[bool, str]
        expired = self.expired

        scopes = self.scopes

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "value": value,
                "sub": sub,
                "scope": scope,
                "exp": exp,
                "createdAt": created_at,
                "updatedAt": updated_at,
                "expired": expired,
                "scopes": scopes,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        value = d.pop("value")

        sub = d.pop("sub")

        scope = d.pop("scope")

        def _parse_exp(data: object) -> Union[None, datetime.datetime]:
            if data is None:
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                exp_type_0 = isoparse(data)

                return exp_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, datetime.datetime], data)

        exp = _parse_exp(d.pop("exp"))

        created_at = isoparse(d.pop("createdAt"))

        updated_at = isoparse(d.pop("updatedAt"))

        def _parse_expired(data: object) -> Union[bool, str]:
            return cast(Union[bool, str], data)

        expired = _parse_expired(d.pop("expired"))

        scopes = cast(List[str], d.pop("scopes"))

        token_response = cls(
            id=id,
            value=value,
            sub=sub,
            scope=scope,
            exp=exp,
            created_at=created_at,
            updated_at=updated_at,
            expired=expired,
            scopes=scopes,
        )

        token_response.additional_properties = d
        return token_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="DabihInfoBrandingContact")


@_attrs_define
class DabihInfoBrandingContact:
    """
    Attributes:
        phone (str):
        country (str):
        state (str):
        city (str):
        zip_ (str):
        street (str):
        email (str):
        name (str):
    """

    phone: str
    country: str
    state: str
    city: str
    zip_: str
    street: str
    email: str
    name: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        phone = self.phone

        country = self.country

        state = self.state

        city = self.city

        zip_ = self.zip_

        street = self.street

        email = self.email

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "phone": phone,
                "country": country,
                "state": state,
                "city": city,
                "zip": zip_,
                "street": street,
                "email": email,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        phone = d.pop("phone")

        country = d.pop("country")

        state = d.pop("state")

        city = d.pop("city")

        zip_ = d.pop("zip")

        street = d.pop("street")

        email = d.pop("email")

        name = d.pop("name")

        dabih_info_branding_contact = cls(
            phone=phone,
            country=country,
            state=state,
            city=city,
            zip_=zip_,
            street=street,
            email=email,
            name=name,
        )

        dabih_info_branding_contact.additional_properties = d
        return dabih_info_branding_contact

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

from ..models.job_status import JobStatus

T = TypeVar("T", bound="Job")


@_attrs_define
class Job:
    """
    Attributes:
        job_id (str):
        status (JobStatus):
    """

    job_id: str
    status: JobStatus

    def to_dict(self) -> Dict[str, Any]:
        job_id = self.job_id

        status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "jobId": job_id,
                "status": status,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        job_id = d.pop("jobId")

        status = JobStatus(d.pop("status"))

        job = cls(
            job_id=job_id,
            status=status,
        )

        return job



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="FileDecryptionKey")


@_attrs_define
class FileDecryptionKey:
    """
    Attributes:
        mnemonic (str): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        key (str): The AES-256 encryption key used to encrypt and decrypt datasets.
            base64url encoded
    """

    mnemonic: str
    key: str

    def to_dict(self) -> Dict[str, Any]:
        mnemonic = self.mnemonic

        key = self.key

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "mnemonic": mnemonic,
                "key": key,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        mnemonic = d.pop("mnemonic")

        key = d.pop("key")

        file_decryption_key = cls(
            mnemonic=mnemonic,
            key=key,
        )

        return file_decryption_key



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define

if TYPE_CHECKING:
    from ..models.inode import Inode


T = TypeVar("T", bound="InodeSearchResults")


@_attrs_define
class InodeSearchResults:
    """
    Attributes:
        is_complete (bool):
        inodes (List['Inode']): The list of inodes that match the search query
    """

    is_complete: bool
    inodes: List["Inode"]

    def to_dict(self) -> Dict[str, Any]:
        is_complete = self.is_complete

        inodes = []
        for inodes_item_data in self.inodes:
            inodes_item = inodes_item_data.to_dict()
            inodes.append(inodes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "isComplete": is_complete,
                "inodes": inodes,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.inode import Inode

        d = src_dict.copy()
        is_complete = d.pop("isComplete")

        inodes = []
        _inodes = d.pop("inodes")
        for inodes_item_data in _inodes:
            inodes_item = Inode.from_dict(inodes_item_data)

            inodes.append(inodes_item)

        inode_search_results = cls(
            is_complete=is_complete,
            inodes=inodes,
        )

        return inode_search_results



from typing import Any, Dict, Type, TypeVar

from attrs import define as _attrs_define

T = TypeVar("T", bound="InodeSearchBody")


@_attrs_define
class InodeSearchBody:
    """
    Attributes:
        query (str):
    """

    query: str

    def to_dict(self) -> Dict[str, Any]:
        query = self.query

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "query": query,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        query = d.pop("query")

        inode_search_body = cls(
            query=query,
        )

        return inode_search_body



from dabih.dabih_client.dabih_api_client.api.filesystem import (
    list_home,
    list_files,
    file_info,
    list_inodes,
    list_parents, 
    search_fs,
    search_cancel,
    search_results
)
from dabih.dabih_client.dabih_api_client.models.inode_search_body import InodeSearchBody
from .json import decode_json   
from .util import check_status
from ..logger import dbg, warn, log, error

import time
import signal
import sys

__all__ = ["list_home_func", "list_files_func", "get_file_info", "search_func"]

def list_home_func(client):
    
    answer = list_home.sync_detailed(client=client)
    check_status(answer)
    response_json = decode_json(answer.content)
    
    def print_structure(items, indent=0):
        for item in items:
            print(f"{'    ' * indent}{item['name']:<30}\t{item['mnemonic']}")
            if 'children' in item:
                print_structure(item['children'], indent + 1)

    parents = response_json.get('parents', [])
    children = response_json.get('children', [])

    parent_map = {parent['id']: parent for parent in parents}
    for child in children:
        parent_id = child['parentId']
        if parent_id in parent_map:
            if 'children' not in parent_map[parent_id]:
                parent_map[parent_id]['children'] = []
            parent_map[parent_id]['children'].append(child)

    print("Home Directory Structure:")
    print_structure(parents)

    return None

def list_files_func(mnemonic, client):

    answer = list_files.sync_detailed(client=client, mnemonic=mnemonic)
    check_status(answer)
    return answer
    
def get_file_info(mnemonic, client):
    answer = file_info.sync_detailed(client = client, mnemonic = mnemonic)
    check_status(answer, context= "file_info")
    file = decode_json(answer.content)
    try:
        data = file["data"]
    except KeyError:
        error(f"File {mnemonic} not found, please check the mnemonic")
        sys.exit(0)
    uid = data["uid"]
    size = data["size"]
    key_list = file["keys"]
    return data, uid, key_list, size

def search_fs_func(client, query):
    search_body = InodeSearchBody(query)
    dbg(f"Search Body: {search_body}")
    answer = search_fs.sync_detailed(client = client, body = search_body)
    check_status(answer)
    decoded_answer = decode_json(answer.content)
    job_id = decoded_answer["jobId"]
    return job_id

def search_results_func(client, job_id):
    answer = search_results.sync_detailed(client = client, job_id = job_id)
    check_status(answer)
    return decode_json(answer.content)

def search_cancel_func(client, job_id):
    log("Cancelling current search.")
    answer = search_cancel.sync_detailed(client = client, job_id = job_id)
    check_status(answer)
    sys.exit(0)
    
def search_func(client, query):
    log("Searching...")
    job_id = search_fs_func(client, query)

    def signal_handler(sig, frame):
        search_cancel_func(client, job_id)
    signal.signal(signal.SIGINT, signal_handler)

    results = []

    while True:
        answer = search_results_func(client, job_id)
        results.append(answer["inodes"])

        if answer["isComplete"] is True:
            dbg(results)
            log("Search Results:")
            if results[0] == []:
                log(f"No matching files found for query: {query}")
            else:
                for result in results:
                    for inode in result:
                        name = inode.get("name")
                        mnemonic = inode.get("mnemonic")
                        created_at = inode.get("createdAt")
                        data = inode.get("data", {})
                        if data: 
                            size = data.get("size")
                            author = data.get("createdBy")
                            print(f"Name: {name}\n\tMnemonic: {mnemonic}\n\tCreated By: {author}\n\tCreated At: {created_at}\n\tSize: {size} MiB")
                        else:
                            print(f"Name: {name}\n\tMnemonic: {mnemonic}\n\tCreated At: {created_at}")

            break
        time.sleep(0.3)



from enum import IntEnum


class InodeType(IntEnum):
    VALUE_0 = 0
    VALUE_1 = 1
    VALUE_2 = 2
    VALUE_10 = 10
    VALUE_11 = 11
    VALUE_12 = 12

    def __str__(self) -> str:
        return str(self.value)



import json
from ..logger import error

__all__=["decode_json", "print_json"]

def decode_json(answer):
    if not answer:
        error("Empty server response; please check if server is reachable")
        raise ValueError("No server response; please check if server is reachable")
    
    answer = answer.decode("utf-8")
    string = json.loads(answer)
    return string

def print_json(json_str):
    decoded_answer = decode_json(json_str)
    print(json.dumps(decoded_answer, indent=2))



from typing import Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define

T = TypeVar("T", bound="TokenAddBody")


@_attrs_define
class TokenAddBody:
    """
    Attributes:
        scopes (List[str]): The array of scopes the token should have
        lifetime (Union[None, int]): The time in seconds the token should be valid for
            If null the token will never expire
    """

    scopes: List[str]
    lifetime: Union[None, int]

    def to_dict(self) -> Dict[str, Any]:
        scopes = self.scopes

        lifetime: Union[None, int]
        lifetime = self.lifetime

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "scopes": scopes,
                "lifetime": lifetime,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        scopes = cast(List[str], d.pop("scopes"))

        def _parse_lifetime(data: object) -> Union[None, int]:
            if data is None:
                return data
            return cast(Union[None, int], data)

        lifetime = _parse_lifetime(d.pop("lifetime"))

        token_add_body = cls(
            scopes=scopes,
            lifetime=lifetime,
        )

        return token_add_body



from typing import Any, Dict, Type, TypeVar, Union

from attrs import define as _attrs_define

from ..types import UNSET, Unset

T = TypeVar("T", bound="AddDirectoryBody")


@_attrs_define
class AddDirectoryBody:
    """
    Attributes:
        name (str): The name of the directory
        parent (Union[Unset, str]): mnemonics are human readable unique identifiers for datasets
            mnemonics have the form <random adjective>_<random first name> Example: happy_jane.
        tag (Union[Unset, str]): A custom searchable tag for the directory
    """

    name: str
    parent: Union[Unset, str] = UNSET
    tag: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        parent = self.parent

        tag = self.tag

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )
        if parent is not UNSET:
            field_dict["parent"] = parent
        if tag is not UNSET:
            field_dict["tag"] = tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        parent = d.pop("parent", UNSET)

        tag = d.pop("tag", UNSET)

        add_directory_body = cls(
            name=name,
            parent=parent,
            tag=tag,
        )

        return add_directory_body



from dabih.dabih_client.dabih_api_client.api.token import (
    token_info,
    add_token,
    list_tokens,
    remove_token, 
    )
from ..logger import dbg, warn, log, error
from .util import check_status
from .json import decode_json

__all__= ["get_token_info", "check_token_validity"]


def get_token_info(client):
    answer = token_info.sync_detailed(client=client)
    check_status(answer)
    info = decode_json(answer.content)
    log(f"Current Token is authorized for User {info['sub']} with the Scopes: {info['scopes']}")
    return None

def check_token_validity(client):
    answer = token_info.sync_detailed(client=client) 
    if check_status(answer):
        return True
    else:
        return False


from dabih.dabih_client.dabih_api_client import AuthenticatedClient

import os
import sys
import yaml
import httpx
import glob
from pathlib import Path
from dabih.logger import dbg, error, warn

__all__ = ["get_client", "find_pem_files", "get_token_and_base_url"]


# Default location for config.yaml file
DEFAULT_CONFIG_DIR = Path.home() / ".config" / "dabih"


def get_config_directory():
    xdg_config_home = os.getenv("XDG_CONFIG_HOME")
    if xdg_config_home:
        config_dir = Path(xdg_config_home) / "dabih"
    else:
        config_dir = DEFAULT_CONFIG_DIR

    if not config_dir.exists():
        error(f"The config directory does not exist: {config_dir}; please create it. Default config_dir is home/.config/dabih or XDG_CONFIG_HOME/dabih (see README.md)")
        sys.exit(0)

    dbg(f"Using config directory: {config_dir}")
    return config_dir


def find_pem_files(config_dir):
    pem_files = glob.glob(str(config_dir / "**" / "*dabih*.pem"), recursive=True)
    if not pem_files:
        warn(f"No dabih PEM files found in config directory: {config_dir}\n Private key is required for downloading files.")
        pem_files = glob.glob(str(config_dir / "**" / "**.pem"), recursive=True)
        dbg(pem_files)
        if pem_files:
            warn(f"Found non-dabih PEM files: {pem_files}. Dabih private keys should contain 'dabih' in filename")
            del pem_files[:]
    else:
        dbg(f"Found PEM files: {pem_files}")
    return pem_files


def load_config(config_dir):
    config_file = config_dir / "config.yaml"
    if config_file.exists():
        try:
            with open(config_file, "r") as f:
                config = yaml.safe_load(f)
                return config
        except yaml.YAMLError as e:
            error(f"Error parsing config file: {e}. Plese check the config file at path: {config_file}")
            sys.exit(0)
    else:
        error(f"Config file does not exist: {config_file} at path: {config_dir}; please create a config.yaml as described in README.md")
        sys.exit(0)


def get_token_and_base_url(config):
    try:
        base_url = config.get("base_url")
        token = config.get("token")
    except KeyError as e:
        error(f"Missing key in config file: {e}")
        raise KeyError(f"Missing key in config file: {e}")
    return base_url, token

def get_client(test=None):
    config_dir = get_config_directory()
    config = load_config(config_dir)
    base_url, token = get_token_and_base_url(config)
    if not "/api/v1" in base_url:
        base_url = base_url+"/api/v1"
    pem_files = find_pem_files(config_dir)

    client = AuthenticatedClient(base_url=base_url, token=token)
    return client, pem_files



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.dabih_info_branding_admin import DabihInfoBrandingAdmin
    from ..models.dabih_info_branding_contact import DabihInfoBrandingContact
    from ..models.dabih_info_branding_department import DabihInfoBrandingDepartment
    from ..models.dabih_info_branding_organization import DabihInfoBrandingOrganization


T = TypeVar("T", bound="DabihInfoBranding")


@_attrs_define
class DabihInfoBranding:
    """
    Attributes:
        organization (DabihInfoBrandingOrganization):
        department (DabihInfoBrandingDepartment):
        contact (DabihInfoBrandingContact):
        admin (DabihInfoBrandingAdmin):
    """

    organization: "DabihInfoBrandingOrganization"
    department: "DabihInfoBrandingDepartment"
    contact: "DabihInfoBrandingContact"
    admin: "DabihInfoBrandingAdmin"
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        organization = self.organization.to_dict()

        department = self.department.to_dict()

        contact = self.contact.to_dict()

        admin = self.admin.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "organization": organization,
                "department": department,
                "contact": contact,
                "admin": admin,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.dabih_info_branding_admin import DabihInfoBrandingAdmin
        from ..models.dabih_info_branding_contact import DabihInfoBrandingContact
        from ..models.dabih_info_branding_department import DabihInfoBrandingDepartment
        from ..models.dabih_info_branding_organization import DabihInfoBrandingOrganization

        d = src_dict.copy()
        organization = DabihInfoBrandingOrganization.from_dict(d.pop("organization"))

        department = DabihInfoBrandingDepartment.from_dict(d.pop("department"))

        contact = DabihInfoBrandingContact.from_dict(d.pop("contact"))

        admin = DabihInfoBrandingAdmin.from_dict(d.pop("admin"))

        dabih_info_branding = cls(
            organization=organization,
            department=department,
            contact=contact,
            admin=admin,
        )

        dabih_info_branding.additional_properties = d
        return dabih_info_branding

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
